function [Average,Standard_deviation,Median,IQR,quartiles,max_num, min_num, ks] = calcmetrics(matrix, alpha, node_number, fig_title, x_label, font_size, print_results)
%UNTITLED Summary of this function goes here
%   Detailed explanation goes here

    % If dibujar_ks is not provided, it is set to 0

    format long;

    if (nargin < 6)
        font_size = 16;
    end

    if (nargin < 7)
        print_results = 2;
    end
    
    if (node_number == NODE.ALL) 
        node_number = 1;
        
        [n,m]=size(matrix);
        matrix = reshape(matrix, [n*m,1]); 
    end
  
    Average = mean(matrix);
    [CI_INF, CI_SUP, CI_REL] = confidence_interval(matrix, alpha);
    Standard_deviation = std(matrix);
    Median = median(matrix);
    quartiles = prctile(matrix, [25, 50 ,75]);
    IQR = quartiles(3) - quartiles(1);
    max_num = max(matrix);
    min_num = min(matrix);

    [ks_x, ks_y]=ksden(matrix, fig_title, x_label, font_size);
    ks = zeros(length(ks_x), 2);
    ks(:,1) = ks_x;
    ks(:,2) = ks_y;
  
    if(print_results == PRINT_RESULTS.DISPLAY)
        disp 'Mean:'; disp(Average(node_number));
        disp 'CI: '; disp([CI_INF, CI_SUP, CI_REL]);
        disp 'Std:'; disp(Standard_deviation(node_number));
        disp 'Median:'; disp(Median(node_number));
        disp 'IQR:'; disp(IQR(node_number));
        disp 'Percentiles 25, 50, 75:'; disp(quartiles(:,node_number));
        disp 'Max'; disp(max_num(node_number));
        disp 'Min'; disp(min_num(node_number));
    else
        if (print_results == PRINT_RESULTS.FILE)
            fid = fopen([fig_title '_data.txt'],'w+');
            
            fprintf(fid, 'Mean: %12.8f\n', Average(node_number));
            fprintf(fid, 'CI: %12.8f\n', [CI_INF, CI_SUP, CI_REL]);
            fprintf(fid, 'Std: %12.8f\n', Standard_deviation(node_number));
            fprintf(fid, 'Median: %12.8f\n', Median(node_number));
            fprintf(fid, 'IQR: %12.8f\n', IQR(node_number));
            fprintf(fid, 'Percentiles 25, 50, 75: %12.8f\n', quartiles(:,node_number));
            fprintf(fid, 'Max: %12.8f\n', max_num(node_number));
            fprintf(fid, 'Min: %12.8f\n', min_num(node_number));

            fclose(fid);
        end
    end
    
    histograma(matrix, fig_title, x_label, font_size);
end



function [x,y] = ksden(matrix, fig_title, x_label, font_size)

  [m,n] = size(matrix);
  x = zeros(numPoints,n);
  y = zeros(numPoints, n);
  
  if (isempty(fig_title) == 0)
      for i = 1:n
          [y(:,i),x(:,i)] = ksdensity(matrix(:,i),'support',[0,1],'npoints', CONSTANTS.NUM_KS_POINTS);
          plot_title(x(:,i), y(:,i), [fig_title, ' - KS-density (instance ', num2str(i), ')'], x_label, 'Density', font_size);
      end
  else
      for i = 1:n
          [y(:,i),x(:,i)] = ksdensity(matrix(:,i),'support',[0,1],'npoints', CONSTANTS.NUM_KS_POINTS);
      end
  end
 
end

function histograma(matrix, fig_title, x_label, font_size)
  
  [m,n] = size(matrix);

  for i = 1:n
      [f,x] = hist(matrix(:,i),30);
      bar_title(x, f, [fig_title, ' - histogram (instance ', num2str(i), ')'], x_label, 'Occurrences', font_size);
      saveas(gcf, fig_title, 'fig');
      saveas(gcf, fig_title, 'png');
      saveas(gcf, fig_title, 'pdf');
      close;
  end

end
