function [ lim_inf, lim_sup, rel_error ] = confidence_interval( data, alpha )
% Usage: [ lim_inf, lim_sup, rel_error ] = confidence_interval( data, alpha )
%
% Inputs: 
%   data: vector to analyze
%   alpha: error of the confidence interval (usually 0.05)
%
% Output:
%   lim_inf: lowest value of the confidence interval
%   lim_sup: upper value of the confidence interval
%   rel_error: confidence interval error relative to the data average

num_samples = length(data);
t_student = tinv(1 - alpha*0.5, num_samples -1);
average = mean(data);
std_data = std(data);

ci_error = t_student * std_data / sqrt(num_samples);
lim_sup = average + ci_error;
lim_inf = average - ci_error;

rel_error = ci_error / average;

end

