function run_analysis(path, result_filedir, num_samples, an_offset)
% Performs the analysis of one single scenario (including all AN/PN values)
%   path: path to the root of the folder that contains the scenario results. The path cannot be the system root.
%   result_filedir: name of the folder that contains the filtered results
%      for each simulation of the scenario (i.e. with the specific number of
%      active nodes)
%   num_samples: number of samples for each variable. It is CONSTANTS.ALL
%      by default, which means that all the samples are taken into account.
%   an_offset: Optional value. It indicates the first simulation to
%      consider in the scenario. It is 0 by default.


    if (nargc < 4)
        an_offset = 0;
    end

    d = dir(path);
    
    % Setting up output variables
    num_nodes = d.length - 2;
    active_nodes = an_offset+1;
    passive_nodes = num_nodes - active_nodes
    
    num_vars = 8;
    avg = zeros(num_vars, num_nodes);
    std = zeros(num_vars, num_nodes);
    md = zeros(num_vars, num_nodes);
    IQR = zeros(num_vars, num_nodes);
    quartiles = zeros(num_vars, num_nodes, 3);
    ks = zeros(num_vars. num_nodes, CONSTANTS.NUM_KS_POINTS,2); 
    
    % Read data
    AP_pos = [0 0];
    j = 1;
    for i=3:length(d) 
        if (d(i).isdir)
            % It's a directory but not current nor parent
            if (ctrcmp(d(i).name, result_filedir))
                [ rtt_mac, rtt_mngt, tdoa, error_rtt_mac, error_rtt_mngt, error_tdoa, dist_an, dist_pn, collisions_an, collisions_pn] = read_data( path , numsamples, active_nodes, passive_nodes);
       
                compute_metrics(VARIABLE.RTT_MAC, j, rtt_mac, 0.05, NODE.ALL, ['RTT MAC ' active_nodes ' active nodes'], 'time (s)', 16, PRINT_RESULTS.FILE);
                compute_metrics(VARIABLE.RTT_MNGT, j, rtt_mngt, 0.05, NODE.ALL, ['RTT MNGT ' active_nodes ' active nodes'], 'time (s)', 16, PRINT_RESULTS.FILE);
                compute_metrics(VARIABLE.TDOA, j, tdoa, 0.05, NODE.ALL, ['TDOA ' passive_nodes ' passive nodes'], 'time (s)', 16, PRINT_RESULTS.FILE);
                compute_metrics(VARIABLE.ACTIVE_NODE_DISTANCE, j, dist_an, 0.05, NODE.ALL, ['Distance ' active_nodes ' active nodes'], 'distance (m)', 16, PRINT_RESULTS.FILE);
                compute_metrics(VARIABLE.PASSIVE_NODE_DISTANCE, j, dist_pn, 0.05, NODE.ALL, ['Distance ' passive_nodes ' active nodes'], 'distance (m)', 16, PRINT_RESULTS.FILE);
                compute_metrics(VARIABLE.COLLISION_ACTIVE_NODES, j, collisions_an, 0.05, NODE.ALL, ['Collision (AN) with ' active_nodes ' active nodes'], 'collisions', 16, PRINT_RESULTS.FILE);
                compute_metrics(VARIABLE.COLLISION_PASSIVE_NODES, j, collisions_pn, 0.05, NODE.ALL, ['Collision (AN) with ' active_nodes ' active nodes'], 'collisions', 16, PRINT_RESULTS.FILE);
                compute_metrics(VARIABLE.RTT_MAC_ERROR, j, rtt_mac_error, 0.05, NODE.ALL, ['RTT MAC (error) with ' active_nodes ' active nodes'], 'time (s)', 16, PRINT_RESULTS.FILE);
                compute_metrics(VARIABLE.RTT_MNGT_ERROR, j, rtt_mngt_error, 0.05, NODE.ALL, ['RTT MNGT (error) with ' active_nodes ' active nodes'], 'time (s)', 16, PRINT_RESULTS.FILE);
                compute_metrics(VARIABLE.TDOA_ERROR, j, tdoa_error, 0.05, NODE.ALL, ['TDOA (error) ' active_nodes ' active nodes'], 'time (s)', 16, PRINT_RESULTS.FILE);
                
                % updating the counters
                j=j+1;
                active_nodes=active_nodes+1;
                passive_nodes=passive_nodes-1;
                                
            else
                run_analysis(an, pn, [path '/' d(i).name], num_samples);
            end
        else
            
        end
    end

    function compute_metrics(variable, j, matrix, alpha, node_number, fig_title, x_label, font_size, print_results)
        [avg(variable, j), std(variable, j), md(variable, j), IQR(variable, j), quartiles(variable, j), ks(variable, j, :)] = calcmetrics(matrix, alpha, node_number, fig_title, x_label, font_size, print_results);    
    end

end

function output = compute_error_rtt(v, AP_pos, positions_an) 
    l = length(AP_pos);

    dist = positions_an - ones(l, 1) * AP_pos;
    actual_RTT = sqrt(sum(power(dist,2),2)) / CONTANTS.SPEED_OF_LIGHT;
    
    output = v - actual_RTT;
end

function output = compute_error_tdoa(v, AP_pos, positions_an, positions_pn) 
% positions_an: cell vector, where each cell is a matrix nx2
% positions_an: cell vector, where each cell is a matrix nx2

    ppos = positions_pn{i};
    apos = positions_an{j}
    
    for k=1:length(apos)
        
    end
end
